#ifndef _CGUY_CPP_
#define _CGUY_CPP_

#include "SDL.h"
#include "SDL_opengl.h"
#include <math.h>

// Local Includes
#include "common.h"
#include "texture.h"

#include "gfx.h"
#include "cGuy.h"
#include "cBlock.h"
#include "Pieces.h"
#include "game.h"
#include "sound.h"

#define pixelsizex 0.00390625
#define pixelsizey 0.0078125
#define spritewidth 0.09375
#define spriteheight 0.25

cGuy::cGuy()
{
	Init(0, 11);

	sprite = GetTexture("island-guy.dds", 0);

	return;
}

void cGuy::Init(int x, int y)
{
	speed = 1.4f;
	xrel = 0;
	yrel = 0;

	SetMessage("What a lovely day!", 25);

	State = 0;
	frameNo = 0;
	Dead = 0;

	MovingLeft = 0;

	gridlocX = x;
	gridlocY = y;	

	return;
}

void cGuy::SetMessage(char * newmsg, float length)
{
	strcpy(message, newmsg);
	messageCounter = length;

	return;
}

void cGuy::Draw()
{
	static float texlocx, texlocy;

	texlocx = (frameNo * 24) * pixelsizex;

	if (State == 0)
		texlocy = 0;
	else if (State == 1)
		texlocy = 32 * pixelsizey;
	else if (State == 2)
		texlocy = 64 * pixelsizey;
	else if (State == 3)
		texlocy = 64 * pixelsizey;
	else if (State == 4)
		texlocy = 96 * pixelsizey;

	glPushMatrix();

		glTranslatef(240 + (gridlocX * 32), 56 + (gridlocY * 32), 0.0f);

		//glDisable(GL_TEXTURE);
		glLineWidth(3.0f);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glBegin(GL_LINE_STRIP);

			glVertex3i(0,0,0);
			glVertex3i(32,0,0);
			glVertex3i(32,32,0);
			glVertex3i(0,32,0);
			glVertex3i(0,0,0);

		glEnd();
		glLineWidth(1.0f);
		glColor3f(1,1,1);

		//glEnable(GL_TEXTURE);

	glPopMatrix();

	glPushMatrix();

		glBindTexture(GL_TEXTURE_2D, sprite->textureID);
		
		glTranslatef(locX, locY + 2, 0);

		if (MovingLeft == 0)
		{
			//Begin rendering quads
			glBegin(GL_QUADS);
			
				glTexCoord2f(texlocx, texlocy);
				glVertex3i(0, 0, 0);
				glTexCoord2f(texlocx, texlocy + spriteheight);
				glVertex3i(0, 32, 00);
				glTexCoord2f(texlocx + spritewidth, texlocy + spriteheight);
				glVertex3i(24, 32, 0);
				glTexCoord2f(texlocx + spritewidth, texlocy);
				glVertex3i(24, 0, 0);

			glEnd();
		}
		else
		{
				//Begin rendering quads
			glBegin(GL_QUADS);
			
				glTexCoord2f(texlocx + spritewidth, texlocy);
				glVertex3i(0, 0, 0);
				glTexCoord2f(texlocx + spritewidth, texlocy + spriteheight);
				glVertex3i(0, 32, 00);
				glTexCoord2f(texlocx, texlocy + spriteheight);
				glVertex3i(24, 32, 0);
				glTexCoord2f(texlocx, texlocy);
				glVertex3i(24, 0, 0);

			glEnd();
		}
	
	glPopMatrix();

	if (messageCounter > 0)
		gfx_DrawTalkyBubble(message, locX, locY - 32);

	return;
}

void cGuy::CheckForFall()
{
	if (State == 0 || State == 1 || State == 2)
	{
		if ((gridlocY + 1) < 12)
		{
			if (field[gridlocX][gridlocY + 1].Active == 0)
			{
				State = 3;
			}
		}
	}

	return;
}

void cGuy::Compute()
{
	static float framecounter;

	if (messageCounter > 0)
		messageCounter -= 0.5f;

	if (exclaimCounter > 0)
		exclaimCounter -= 0.5f;

	if (Dead == 0 && ShowWinScreen == FALSE)
	{
		if (MovingLeft)
		{
			if (State == 1 || State == 0)
			{
				xrel -= speed;

				if (xrel < -8)
				{
					if (gridlocX == 0)
					{
						MovingLeft = 0;
					}
					//else if (field[gridlocX - 1][gridlocY].Active == 1)
					else if (IsFreeBlockWithPiece(gridlocX - 1, gridlocY) == FALSE)
					{
						//if (field[gridlocX - 1][gridlocY - 1].Active == 0)
						if (IsFreeBlockWithPiece(gridlocX - 1, gridlocY - 1))
						{
							State = 4; // jumping
							frameNo = 0;
						}
						else
							MovingLeft = 0;
					}
					else
					{
						if ((gridlocY + 1) < 12 && gridlocX >= 0 && gridlocX < 10)
						{
							// check for fall
							//if (field[gridlocX - 1][gridlocY + 1].Active == 0)
							if (IsFreeBlockWithPiece(gridlocX - 1, gridlocY + 1))
							{
								if ((gridlocY + 2) < 12)
								{
									// make sure it's only a short fall :)
									//if (field[gridlocX - 1][gridlocY + 2].Active == 1)
									if (IsFreeBlockWithPiece(gridlocX - 1, gridlocY + 2) == FALSE)
									{
										// fall time!
										State = 3;
										gridlocX -= 1;
										xrel += 32;
									}
									else
									{
										MovingLeft = 0;
										SetMessage("too far down to jump", 30);
									}
								}
								else
								{
									State = 3;
									gridlocX -= 1;
									xrel += 32;
								}
							}
							else
							{
								gridlocX -= 1;
								xrel += 32;
							}
						}
						else
						{
							gridlocX -= 1;
							xrel += 32;
						}
					}
				}
			}
		}
		else
		{
			if (State == 1 || State == 0)
			{
				xrel += speed;

				if (xrel > 20)
				{
					if (gridlocX == 9)
					{
						MovingLeft = 1;
					}
					//else if (field[gridlocX + 1][gridlocY].Active == 1)
					else if (IsFreeBlockWithPiece(gridlocX + 1, gridlocY) == FALSE)
					{
						// check for jump
						//if (field[gridlocX + 1][gridlocY - 1].Active == 0)
						if (IsFreeBlockWithPiece(gridlocX + 1, gridlocY - 1))
						{
							State = 4; // jumping
							frameNo = 0;
						}
						else
							MovingLeft = 1;
					}
					else
					{
						if ((gridlocY + 1) < 12 && gridlocX >= 0 && gridlocX < 10)
						{
							// check for fall
							//if (field[gridlocX + 1][gridlocY + 1].Active == 0)
							if (IsFreeBlockWithPiece(gridlocX + 1, gridlocY + 1))
							{
								if ((gridlocY + 2) < 12)
								{
									// make sure it's only a short fall :)
									//if (field[gridlocX + 1][gridlocY + 2].Active == 1)
									if (IsFreeBlockWithPiece(gridlocX + 1, gridlocY + 2) == FALSE)
									{
										// fall time!
										State = 3;
										gridlocX += 1;
										xrel -= 32;
									}
									else
									{
										MovingLeft = 1;
										SetMessage("don't wanna jump that far down", 30);
									}
								}
								else
								{
									State = 3;
									gridlocX += 1;
									xrel -= 32;
								}
							}
							else
							{
								gridlocX += 1;
								xrel -= 32;
							}
						}
						else
						{
							gridlocX += 1;
							xrel -= 32;
						}
						
						
					}
				}
			}
		}

		if (State == 2 && ShowWinScreen == FALSE) // flying up
		{
			yrel -= (speed * 2);

			if (yrel < -32)
			{
				yrel = 0;

				gridlocY -= 1;

				if (MovingLeft)
				{
					gridlocX -= 1;
					xrel += 32;
				}
				else
				{
					gridlocX += 1;
					xrel -= 32;
				}

				State = 0;
			}
		}
		else if (State == 3 && ShowWinScreen == FALSE) // falling down
		{
			yrel += (speed * 2);

			if (yrel > 32)
			{
				yrel = 0;
				gridlocY += 1;

				// solid ground?
				if ((gridlocY + 1) < 12)
				{
					if (field[gridlocX][gridlocY + 1].Active == 1)
					{
						State = 0;
						snd_PlaySound("snd/land.ogg");
					}
				}
				else
				{
					State = 0;	
					snd_PlaySound("snd/land.ogg");
				}
			}
		}
		
		if (1) // State == 0 || State == 1
		{
			framecounter += speed;

			if (framecounter > 3)
			{
				frameNo += 1;

				if (State == 2)
				{
					if (frameNo == 4)
					{
						frameNo = 0;
					}
				}
				else if (State == 3)
				{
					if (frameNo == 8)
					{
						frameNo = 5;
					}
				}
				else if (State == 4 && frameNo == 8)
				{
					State = 2; // flying up
					snd_PlaySound("snd/jump.ogg");
					frameNo = 0;
				}
				else if (frameNo == 8)
				{
					frameNo = 0;
				}
			}
		}

		locX = 240 + (gridlocX * 32) + xrel;
		locY = 56 + (gridlocY * 32) + yrel;

		if (State == 0 && (448 - WaterLevel) < locY)
		{
			State = 1;
			SetMessage("water makes me nervous!", 25);
			snd_PlaySound("snd/woah.ogg");
			//exclaimCounter = 60;
		}
		else if (State == 1)
		{
			if ((484 - WaterLevel) < locY)
			{
				Dead = 1;
				SetMessage("guurrkklkblublbl", 50);
				snd_PlaySound("snd/drown.ogg");
				ShowLoseScreen = TRUE;
			}
			else if ((448 - WaterLevel) > locY)
			{
				State = 0;
				SetMessage("phew, that was close!", 25);
			}
		}
	}
	else
	{
		if (ShowWinScreen == FALSE)
			locY += 1;
	}

	return;
}


#endif