#ifndef _COMMON_C_
#define _COMMON_C_

// System Includes
#include <string.h>
#include <time.h>
#include <stdlib.h>
//#include <math.h>
#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include "SDL.h"
#include "SDL_opengl.h"
#include "physfs/physfs.h"

// Local Includes
#include "common.h"
#include "sound.h"
#include "texture.h"

// Globals
int ScreenWidth;
int ScreenHeight;
int GameScreenWidth;
int GameScreenHeight;
int ScaleFactor;
int ScreenBPP;
int Fullscreen;
int maxFPS;
int skipMSPerFrame;
const char *systemVersion = "0.0.1";

// Locals
char userpath[4096];
char logfilename[4096];
char tempdate [10];
char temptime [10];
char datetime [30];

#define PI_DIV_THREE_SIXTY 0.0087266388f

int GetRandomNum(int topend)
{
	// returns an integer number between 0 and topend - 1
	return (rand() % topend);
}

/*
void SetPerspective(GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar)
{
   GLdouble xmin, xmax, ymin, ymax;

   ymax = zNear * tan(fovy * PI_DIV_THREE_SIXTY);
   ymin = -ymax;
   xmin = ymin * aspect;
   xmax = ymax * aspect;

   glFrustum(xmin, xmax, ymin, ymax, zNear, zFar);
   
   return;
}*/

// OS specific (Windows)
int FolderExists(char * file)
{
	DWORD returnvalue;
	returnvalue = GetFileAttributesA(file);

	if (returnvalue == ((DWORD) - 1))
		return FALSE;
 	else
		return TRUE;
}

// OS specific (Windows)
void MakeADirectory(char * dir)
{
	CreateDirectoryA(dir, NULL);

	/*printf("Now creating: ");
	printf(dir);
	printf("\n");*/
	
	return;
}

void InitLog()
{
	// Find the user's profile directory
	char * tempfilename;
	FILE *fp;
	
	// OS Specific (Windows)
	// ---
	size_t len;
	_dupenv_s(&tempfilename, &len, "USERPROFILE");

	if (!tempfilename)
	{
		printf("Cannot find %USERPROFILE% environment variable. Now exiting.\n");
		exit(1);
	}
	// ---
	
	strcpy_s (userpath, 4096, tempfilename);
	strcpy_s (logfilename, 4096, tempfilename);

	free(tempfilename);
	
	// Make sure the whole path to the log file exists
	strcat_s (logfilename, 4096, "\\My Documents");

	if (!FolderExists(logfilename))
	{
		// My Documents is missing!
		MakeADirectory(logfilename);
	}

	strcat_s (logfilename, 4096, "\\My Games");
	
	if (!FolderExists(logfilename))
	{
		// My Documents\My Games is missing!
		MakeADirectory(logfilename);
	}
	
	strcat_s (logfilename, 4096, "\\FydoLD17");
	
	if (!FolderExists(logfilename))
	{
		// My Documents\My Games\DuskSoftware is missing!
		MakeADirectory(logfilename);
	}

	strcat_s (logfilename, 4096, "\\log.txt");

	// See if the log file exists

	fopen_s(&fp, logfilename,"r");

	if (fp)
	{
		// Log file already exists

		// Check file size!
		/*
		int ch = (int) NULL, pos = 0;

		while(ch != EOF)
		{
			ch = fgetc(fp);
			pos += 1;
		}

		fclose(fp);

		//printf("File size is %d", pos);

		if (pos > 100000) // if log file is over 100k
		{
			// delete the log file?
			//printf("File is pretty big!\n");
		}

		*/
	} 
	else 
	{
		// Log file doesn't exist yet
		if (fopen_s(&fp, logfilename, "a") != 0)
		{
			printf("Cannot open log file. Be sure that this game has write rights. Now exiting.\n");
			exit(1);
		}
	}

	fclose(fp);

	return;

}

void SetDateAndTime()
{
	
	_strdate_s(tempdate, 10);
	_strtime_s(temptime, 10);

	strcpy_s (datetime, 30, tempdate);
	strcat_s (datetime, 30, " ");
	strcat_s (datetime, 30, temptime);
	strcat_s (datetime, 30, " - "); 

	return;
}


void Log(const char* message, ...)
{
	// Variables
	va_list args;
	FILE * fp;

	SetDateAndTime();

	va_start(args, message);

	// If debugging, print to the console too!
	#ifdef _DEBUG
		fprintf(stdout, datetime);
		vfprintf(stdout, message, args);
		fprintf(stdout, "\n");
	#endif
	
	// Open up the log file
	
	if (fopen_s(&fp, logfilename, "a") != 0)
	{
		printf("Cannot open log file. Now exiting.\n");
		exit(1);
	}
	
	// Write to the stream
	fprintf(fp, datetime);
	vfprintf(fp, message, args); 
	va_end(args);

	// End the line
	fprintf(fp, "\n");

	// Close file stream
	fclose(fp);
	
	return;	
}

void Shutdown(int code)
{
	Log("Now exiting. Bye!");

	// clear allocated memories
	FreeTextureData();
	QuitSound();
	
	// exit PhysFS
	//PHYSFS_deinit();

	// exit SDL
	SDL_Quit();
	
	// k bye
	exit(code);
}

#endif
