#ifndef _FONT_C_
#define _FONT_C_

//#include <stdio.h>
//#include <string.h>

#include "SDL.h"
#include "SDL_opengl.h"

#include "common.h"
#include "texture.h"
#include "font.h"

// This is the width of a pixel!
#define BASEWIDTH 0.001953125f
// (1.0 / 512.0)

font_t theFont;

int InitFonts()
{
    // Variables!

	float baseWidth = BASEWIDTH;
    float finalHeight = (9.0 / 16.0);
    int currPos = 0;
    int x;

    // font widths!
    //                    ,!,",#,S,%,&,',(,),*,+,,,-,.,/,0,1,2,3,4,5,6,7,8,9,:,;,<,=,>,?,@,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,[,\,],^,_,`,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,{,|,},~
    int charsizes[96] = {3,1,3,5,5,5,5,1,2,2,3,3,2,3,1,3,3,3,3,3,3,3,3,3,3,3,1,2,3,3,3,3,4,5,4,4,5,4,2,4,4,1,2,4,2,5,3,4,4,4,2,3,2,3,3,5,3,4,3,3,3,3,3,3,2,5,4,4,5,4,2,4,4,1,2,4,2,5,3,4,4,4,2,3,2,3,3,5,3,4,3,3,1,3,5}; // for temporarily holding the widths of the characters
	
	TextureRef_t * temp = GetTexture("fontblack.dds", 0);

	theFont.texture = temp;
    // Generate array of FONTCHARS with proper information
    theFont.IntStart = 32;

    for (x = 0; x < 96; x++)
    {
        theFont.Char[x].dx = (float)charsizes[x];
        theFont.Char[x].dy = 9.0;
        theFont.Char[x].tx1 = currPos * baseWidth;
        theFont.Char[x].tx2 = (currPos + charsizes[x]) * baseWidth;
        theFont.Char[x].ty1 = 0.00;
        theFont.Char[x].ty2 = finalHeight;

        //debug
        //fprintf( stdout, "%f, %f, %f, %f, %f, %f\n", theFont.Char[x].dx, theFont.Char[x].dy, theFont.Char[x].tx1, theFont.Char[x].tx2, theFont.Char[x].ty1, theFont.Char[x].ty2);

        currPos += charsizes[x] + 1;
    }

    // TODO: Build display lists for each character?

    return TRUE;
}

int GetFontTextLength(char *String)
{
	static int Length, i;
	static int place = 0;
	static fontchar_t Char;

	place = 0;
	
	Length = (int)strlen(String);

	for (i = 0; i < Length; i++)
    {
		Char = theFont.Char[(int)String[i] - theFont.IntStart];

		place += Char.dx + 1;
	}

	return place;
}

int DrawFontText (const char *String, float x, float y, float z, int UseBillboard, float zoomlevel)
{
    static int Length, i;
    static float place = 0.0f;
    static fontchar_t Char;
	
	place = 0.0f;

	// Return if we don't have a valid font 
    if (theFont.IntStart == 0)
        InitFonts();

    glPushMatrix();

    glTranslatef(x, y, z);

    if (UseBillboard == TRUE)
    {
        //BillboardPush();
        glScalef(0.02f, 0.02f, 0.02f);
    }

    // Scale to zoomlevel
    if (zoomlevel < 1) 
        zoomlevel = 1;
    //glScalef(-3,-3,-3);
    //glScalef(zoomlevel, zoomlevel, zoomlevel);
    // Set the texture
    glBindTexture(GL_TEXTURE_2D, theFont.texture->textureID);

    // Get length of string
    Length = (int)strlen(String);

    //Begin rendering quads
    glBegin(GL_QUADS);

    //Loop through characters
    for (i = 0; i < Length; i++)
    {
        //Get pointer to character
        Char = theFont.Char[(int)String[i] - theFont.IntStart];

        //Specify vertices and texture coordinates
        glTexCoord2f(Char.tx1, Char.ty1);
        glVertex3f(place, 0.0f, 0.0f);
        glTexCoord2f(Char.tx1, Char.ty2);
        glVertex3f(place, Char.dy, 0.0f);
		glTexCoord2f(Char.tx2, Char.ty2);
        glVertex3f(place + Char.dx, Char.dy, 0.0f);
        glTexCoord2f(Char.tx2, Char.ty1);
        glVertex3f(place + Char.dx, 0.0f, 0.0f);

        //Move to next character
        place += Char.dx + 1;
    }

    //Stop rendering quads
    glEnd();

    if (UseBillboard == TRUE)
    {
        //BillboardPop();
    }

    

    glPopMatrix();

	//GLenum er = glGetError();

	return place;
}

#endif