#ifndef _GAME_C_
#define _GAME_C_

// System Includes
#include <windows.h>
#include "SDL.h"
#include "SDL_opengl.h"

// Local Includes
#include "common.h"
#include "sound.h"
#include "input.h"
#include "texture.h"
#include "font.h"
#include "gfx.h"
#include "cGuy.h"
#include "cBlock.h"
#include "Pieces.h"


// Timing Variables
const int TICKS_PER_SECOND = 25;
const int SKIP_TICKS =  40; // 1000 / TICKS_PER_SECOND;
const int MAX_FRAMESKIP = 5;
DWORD timing_next_game_tick;
int timing_loops;
float timing_interpolation;
 
// game stuffs
float WaterLevel = 10;
int GoalHeight = 3;
cBlock **field;
cGuy * guy;
Pieces pieces;

int currPosX, currPosY;
int PieceType, PieceRotation;
//int nextPosX, nextPosY;
int NextPieceType, NextPieceRotation;

float lineClearCounter;
float lineClearYpos;

int LinesCleared;
int LinesRequired;

int CurrentScore;

int ShowWinScreen;
int ShowLoseScreen;

int MenuSelection = 0;
int CurrentLevel = 1;

void ResetGame(int level)
{
	int x;

	lineClearCounter = 0;
	lineClearYpos = 0;
	LinesCleared = 0;
	LinesRequired = level + 1;
	GoalHeight = level + 2;
	CurrentScore = 0;
	ShowWinScreen = 0;
	ShowLoseScreen = 0;
	WaterLevel = 0;

	field = new cBlock*[10];

	for (x = 0; x < 10; x++)
		field[x] = new cBlock[12];

	/*field[3][11].Active = 1;
	field[3][10].Active = 1;
	field[2][11].Active = 1;
	*/

	PieceType = GetRandomNum(7);
	PieceRotation = GetRandomNum(4);
	currPosX = 5 + pieces.GetXInitialPosition(PieceType, PieceRotation);
	currPosY = pieces.GetYInitialPosition(PieceType, PieceRotation);

	NextPieceType = GetRandomNum(7);
	NextPieceRotation = GetRandomNum(4);

	guy->Init(0,11);
	//nextPosX = 

	return;
}

void GetNewPiece()
{
	PieceType = NextPieceType;
	PieceRotation = NextPieceRotation;

	currPosX = 5 + pieces.GetXInitialPosition(PieceType, PieceRotation);
	currPosY = pieces.GetYInitialPosition(PieceType, PieceRotation);

	NextPieceType = GetRandomNum(7);
	NextPieceRotation = GetRandomNum(4);

	return;
}

void DeleteLine(int y)
{
	int j, i;

	// TODO: Some kind of special effect?

	lineClearYpos = y;
	lineClearCounter = 50;
	LinesCleared++;
	CurrentScore += 50;

	// Moves all the upper lines one row down  
	for (j = y; j > 0; j--)
	{
		for (i = 0; i < 10; i++)
		{
			field[i][j] = field[i][j - 1];
		}
	}
}

void DeletePossibleLines ()
{
	int delCount;

	delCount = 0;

	for (int j = 0; j < 12; j++)
	{
		int i = 0;
		while (i < 10)
		{
			if (field[i][j].Active == FALSE) break;
			i++;
		}

		if (i == 10)
		{
			DeleteLine (j);
			delCount++;
		}
	}

	if (delCount > 0)
	{
		if (delCount > 1)
		{
			// multiple lines! drop a fruit?
		}

		snd_PlaySound("snd/clearline.ogg");
	}

	guy->CheckForFall();
}

int IsFreeBlock(int x, int y)
{
	if (field[x][y].Active == 1)
		return FALSE;
	else
		return TRUE;
}

int IsFreeBlockWithPiece(int x, int y)
{
	// check the current piece
	//static int offsetX, offsetY;

	if (field[x][y].Active == 1)
		return FALSE;
	else
	{	

		//offsetX = x - currPosX;
		//offsetY = y - currPosY;

		// check bounds
		//if (offsetX >= 0 && offsetX < 5 && offsetY >= 0 && offsetY < 5)

		for (int i1 = currPosX, i2 = 0; i1 < currPosX + 5; i1++, i2++)
		{
			for (int j1 = currPosY, j2 = 0; j1 < currPosY + 5; j1++, j2++)
			{
				if (i1 == x && j1 == y)
				{
					if (pieces.GetBlockType(PieceType, PieceRotation, j2, i2) != 0)
					{
						return FALSE;
					}
				}
			}
		}
	}

	return TRUE;
}

void StorePiece (int pX, int pY, int pPiece, int pRotation)
{
	int PlaySquish = FALSE;
	int PlayLose = FALSE;

	// Store each block of the piece into the board
	for (int i1 = pX, i2 = 0; i1 < pX + 5; i1++, i2++)
	{
		for (int j1 = pY, j2 = 0; j1 < pY + 5; j1++, j2++)
		{	
			// Store only the blocks of the piece that are not holes
			if (pieces.GetBlockType (PieceType, PieceRotation, j2, i2) != 0)
			{
				// Check bounds
				if (i1 > 9 || i1 < 0 || j1 > 11 || j1 < 0)
				{
					// omg lol game over
					//InputState.NeedToShutDown = TRUE;

					ShowLoseScreen = TRUE;
					//counter = 0;
					lineClearCounter = 0;
					guy->Dead = TRUE;
					guy->SetMessage("i can't handle all those blocks!", 70);
					PlayLose = TRUE;
				}
				else
				{
					field[i1][j1].Active = TRUE;
					field[i1][j1].Type = GetRandomNum(3);

					//check if the guy got squished
					if (guy->gridlocX == i1 && guy->gridlocY == j1)
					{
						// squish!!
						guy->Dead = TRUE;
						guy->State = 1; // angry state
						guy->SetMessage("i got squished!", 30);
						PlaySquish = TRUE;
						ShowLoseScreen = TRUE;
					}
				}
			}
		}
	}

	if (PlaySquish == TRUE)
	{
		snd_PlaySound("snd/squish.ogg");
	}
	else if (PlayLose == TRUE)
	{
		snd_PlaySound("snd/woah.ogg");
	}
	else
	{
		snd_PlaySound("snd/blockland.ogg");
	}

}

int IsPossibleMovement (int pX, int pY, int pPiece, int pRotation)
{
	// Checks collision with pieces already stored in the board or the board limits
	// This is just to check the 5x5 blocks of a piece with the appropiate area in the board
	for (int i1 = pX, i2 = 0; i1 < pX + 5; i1++, i2++)
	{
		for (int j1 = pY, j2 = 0; j1 < pY + 5; j1++, j2++)
		{	
			// Check if the piece is outside the limits of the board
			if (i1 < 0  || i1 > 9 || j1 > 11)
			{
				if (pieces.GetBlockType (pPiece, pRotation, j2, i2) != 0)
					return FALSE;		
			}

			// Check if the piece have collisioned with a block already stored in the map
			if (j1 >= 0)	
			{
				if ((pieces.GetBlockType (pPiece, pRotation, j2, i2) != 0) &&
					(IsFreeBlock (i1, j1) == FALSE))
					return FALSE;
			}
		}
	}

	// No collision
	return TRUE;
}

void UpdateGame()
{
	static int counter;
	static int dropCounter;
	static int inputCounter;

	counter += 1;
	dropCounter += 1;

	if (ShowWinScreen == 0 && ShowLoseScreen == 0)
	{
		if (counter > 3)
		{
			counter = 0;
			WaterLevel += 0.3f;
			CurrentScore += 2;
		}

		if (lineClearCounter > 0)
		{
			lineClearCounter -= 0.5f;
		}

		if (dropCounter > 15)
		{
			dropCounter = 0;

			if (IsPossibleMovement(currPosX, currPosY + 1, PieceType, PieceRotation) == TRUE)
				currPosY++;
			else
			{
				StorePiece(currPosX, currPosY, PieceType, PieceRotation);
				
				DeletePossibleLines();

				GetNewPiece();
			}
		}

		if (inputCounter < 10)
			inputCounter++;

		// player stuff
		if (InputState.MoveLeft == TRUE && inputCounter > 2)
		{
			inputCounter = 0;

			if (IsPossibleMovement(currPosX - 1, currPosY, PieceType, PieceRotation) == TRUE)
				currPosX--;

			//InputState.MoveLeft = FALSE;
		}
		if (InputState.MoveRight == TRUE && inputCounter > 2)
		{
			inputCounter = 0;

			if (IsPossibleMovement(currPosX + 1, currPosY, PieceType, PieceRotation) == TRUE)
				currPosX++;

			//InputState.MoveRight = FALSE;
		}
		if (InputState.MoveDown == TRUE)
		{
			if (IsPossibleMovement(currPosX, currPosY + 1, PieceType, PieceRotation) == TRUE)
				currPosY++;

			//InputState.MoveDown = FALSE;
		}
		if (InputState.TryRotate == TRUE)
		{
			if (IsPossibleMovement (currPosX, currPosY, PieceType, (PieceRotation + 1) % 4) == TRUE)
				PieceRotation = (PieceRotation + 1) % 4;

			InputState.TryRotate = FALSE;
		}
		if (InputState.ESCPressed == TRUE)
		{
			InputState.MenuMode = TRUE;

			InputState.ESCPressed = FALSE;
		}

		
		// Check for win!

		if ((LinesCleared >= LinesRequired) && (guy->gridlocY <= (11 - GoalHeight)) && (guy->State != 3))
		{
			//InputState.NeedToShutDown = TRUE;
			ShowWinScreen = TRUE;
			counter = 0;
			lineClearCounter = 0;
			guy->SetMessage("I am so excited! Yay!", 100);
			
			snd_PlaySound("snd/woohoo.ogg");
		}
	}
	else
	{
		if (lineClearCounter < 100.0f)
			lineClearCounter += 1.8f;

		if (WaterLevel < 800.0f && ShowLoseScreen == TRUE)
			WaterLevel += 1.5f;

		if (InputState.TryRotate == TRUE)
		{
			//lineClearCounter = 0;
			InputState.TryRotate = FALSE;
			
			//ResetGame(CurrentLevel);
			
			if (ShowWinScreen == TRUE)
			{
				CurrentLevel++;
				ResetGame(CurrentLevel);
			}
			else if (ShowLoseScreen == TRUE)
			{
				// end / restart game
				InputState.MenuMode = TRUE;
			}

			ShowWinScreen = FALSE;
			ShowLoseScreen = FALSE;
			
		}
		if (InputState.ESCPressed == TRUE)
		{
			InputState.MenuMode = TRUE;

			InputState.ESCPressed = FALSE;
		}
		//if (ShowWinScreen == 1)
	}

	guy->Compute();

	return;
}

void UpdateMenu()
{
	WaterLevel = 10;

	if (InputState.MoveLeft == TRUE)
	{
		if (MenuSelection == 1)
		{
			MenuSelection = 0;
			snd_PlaySound("snd/switch.ogg");
		}

		InputState.MoveLeft = FALSE;
	}
	if (InputState.MoveRight == TRUE)
	{
		if (MenuSelection == 0)
		{
			MenuSelection = 1;
			snd_PlaySound("snd/switch.ogg");
		}

		InputState.MoveRight = FALSE;
	}
	if (InputState.MoveDown == TRUE)
	{
		
		InputState.MoveDown = FALSE;
	}
	if (InputState.TryRotate == TRUE)
	{
		if (MenuSelection == 0)
		{
			snd_PlaySound("snd/select.ogg");
			snd_StartMusic("snd/music1.ogg");
			CurrentLevel = 0;
			ResetGame(CurrentLevel);
			InputState.MenuMode = FALSE;
		}
		else
		{
			snd_PlaySound("snd/select.ogg");
			InputState.NeedToShutDown = TRUE;
		}

		InputState.TryRotate = FALSE;
	}
	if (InputState.ESCPressed == TRUE)
		{
			InputState.NeedToShutDown = TRUE;

			InputState.ESCPressed = FALSE;
		}

	
	return;
}

// This is adapted from Dewitter's online tutorial on game loop and timing
int MainLoop()
{
	static float tempfloat;
	int x;

	timing_next_game_tick = GetTickCount(); // init timer

	guy = new cGuy();
	//ResetGame();

	while(InputState.NeedToShutDown == FALSE)
	{
		// Call to input.c
		HandleInput();

		timing_loops = 0;
		while((GetTickCount() > timing_next_game_tick) && (timing_loops < MAX_FRAMESKIP))
		{
			if (InputState.MenuMode == TRUE)
			{
				UpdateMenu();
			}
			else
			{
				UpdateGame();
			}

			timing_next_game_tick += SKIP_TICKS;
			timing_loops++;
		}

		//timing_interpolation = GetTickCount() + SKIP_TICKS - timing_next_game_tick;
		// MS's ANSI C compiler won't let me cast ints to floats, so this will have to do... :P
		//tempfloat = SKIP_TICKS;
		//timing_interpolation = timing_interpolation / tempfloat;
		gfx_DrawScene(0);

		SDL_Delay(skipMSPerFrame);
    }

	delete guy;

	if (field)
	{
	for (x = 0; x < 10; x++)
		delete [] field[x];
	delete [] field;
	}

	return 0; // 0 means no errors
}

#endif