#ifndef _GFX_C_
#define _GFX_C_

#include "SDL.h"
#include "SDL_opengl.h"
#include <math.h>

// Local Includes
#include "common.h"
#include "input.h"
#include "texture.h"
#include "font.h"
#include "gfx.h"
#include "cGuy.h"
#include "cBlock.h"
#include "Pieces.h"
#include "game.h"
#include "sound.h"

TextureRef_t * bg;
TextureRef_t * water;
TextureRef_t * talkybubble;
TextureRef_t * block;
TextureRef_t * winlose;
TextureRef_t * symbols;
TextureRef_t * labels;
TextureRef_t * titlebits;

int TexturesLoaded = 0;

void gfx_LoadTextures()
{
	water = GetTexture("water.dds", 0);
	bg = GetTexture("bg1.dds", 0);	
	talkybubble = GetTexture("talkbubble.dds", 0);
	block = GetTexture("blocks.dds", 0);
	winlose = GetTexture("winlose.dds", 0);
	symbols = GetTexture("symbols.dds", 0);
	labels = GetTexture("labels.dds", 0);
	titlebits = GetTexture("title-bits.dds", 0);

	snd_LoadSound("snd/blockland.ogg");
	snd_LoadSound("snd/drown.ogg");
	snd_LoadSound("snd/select.ogg");
	snd_LoadSound("snd/squish.ogg");
	snd_LoadSound("snd/switch.ogg");
	snd_LoadSound("snd/woohoo.ogg");
	snd_LoadSound("snd/woah.ogg");
	snd_LoadSound("snd/jump.ogg");
	snd_LoadSound("snd/land.ogg");
	snd_LoadSound("snd/clearline.ogg");

	snd_LoadSound("snd/intro-music.ogg");
	snd_PlaySound("snd/intro-music.ogg");
	//LoadSound("snd/woohoo.ogg");


	TexturesLoaded = 1;

	return;
}

void gfx_DrawSquare(int x, int y, int R, int G, int B)
{
	int realX, realY;

	realX = (x * 32) + x;
	realY = (y * 32) + y;
	
	// disable textures for a sec
	glDisable(GL_TEXTURE_2D);

	// set color
	glColor3f(R / 255.0f, G / 255.0f, B / 255.0f);

	glBegin(GL_QUADS);
	
        glVertex3f(realX, realY, 0.0f);
        glVertex3f(realX, realY + 32, 0.0f);
        glVertex3f(realX + 32, realY + 32, 0.0f);
        glVertex3f(realX + 32, realY, 0.0f);	
	
	glEnd();

	// reset color back to normal
	glColor3f(1, 1, 1);

	// re-enable textures
	glEnable(GL_TEXTURE_2D);

	return;
}

void gfx_DrawMenuTitle()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, titlebits->textureID);
	
	glTranslatef(144.0f, 50.0f, 0.0f);

	//Begin rendering quads
    glBegin(GL_QUADS);
	
		glTexCoord2i(0, 0);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0, 0.712890625);
        glVertex3i(0, 365, 00);
		glTexCoord2f(1, 0.712890625);
        glVertex3i(512, 365, 0);
        glTexCoord2f(1, 0);
        glVertex3i(512, 0, 0);

	glEnd();
	
	glPopMatrix();

	// draw the selector thingy now

	glPushMatrix();
	
	glBindTexture(GL_TEXTURE_2D, titlebits->textureID);

	glTranslatef(54.0f + (MenuSelection * 256), 140.0f, 0.0f);

	//Begin rendering quads
    glBegin(GL_QUADS);
	
		glTexCoord2f(0, 0.712890625);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0, 1);
        glVertex3i(0, 147, 00);
		glTexCoord2f(1, 1);
        glVertex3i(512, 147, 0);
        glTexCoord2f(1, 0.712890625);
        glVertex3i(512, 0, 0);

	glEnd();

	glPopMatrix();

	return;
}

/*
void gfx_DrawMenuItems()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, titlebits->textureID);
	
	glTranslatef(144.0f, 50.0f, 0.0f);

	//Begin rendering quads
    glBegin(GL_QUADS);
	
		glTexCoord2i(0, 0);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0, 0.2734375);
        glVertex3i(0, 140, 00);
		glTexCoord2f(1, 0.2734375);
        glVertex3i(256, 140, 0);
        glTexCoord2f(1, 0);
        glVertex3i(256, 0, 0);

	glEnd();
	
	glPopMatrix();

	return;
}*/

void gfx_DrawBackground()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, bg->textureID);
	
	//Begin rendering quads
    glBegin(GL_QUADS);
	
		glTexCoord2i(0, 0);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0, 0.9375);
        glVertex3i(0, 480, 00);
		glTexCoord2f(0.78125, 0.9375);
        glVertex3i(800, 480, 0);
        glTexCoord2f(0.78125, 0);
        glVertex3i(800, 0, 0);

	glEnd();
	
	glPopMatrix();

	return;
}

void gfx_DebugCrap()
{
	int x, y;
	static char MouseString[512];

	for (x = 0; x < 10; x++)
	{
		for (y = 0; y < 12; y++)
		{
			sprintf_s(MouseString, 512, "%d, %d", x, y);

			glPushMatrix();
				glTranslatef(240 + (x * 32), 56 + (y * 32), 0.0f);

				DrawFontText(MouseString, 0, 0, 0.0f, FALSE, 1);
			glPopMatrix();
		}
	}
}

void gfx_DrawLines()
{
	int x, y;

	glPushMatrix();

	// disable textures for a sec
	glDisable(GL_TEXTURE_2D);

	glColor4f(1.0f, 1.0f, 1.0f, 0.25f);

	glBegin(GL_LINES);
	
		// outside box
		glVertex3i(240, 440, 0);
		glVertex3i(560, 440, 0);
		glVertex3i(240, 440, 0);
		glVertex3i(240, 56,  0);
		glVertex3i(560, 440, 0);
		glVertex3i(560, 56,  0);
	
		for (x = 0; x < 10; x++)
		{
			glVertex3i(240 + (x * 32), 440, 0);
			glVertex3i(240 + (x * 32), 56,  0);

			//DrawFontText(to_string(x).c_str(), 240 + (x * 32), 445.0f, 0.0f, FALSE, 1);
		}

		for (y = 0; y < 12; y++)
		{
			glVertex3i(240, 56 + (y * 32), 0);
			glVertex3i(560, 56 + (y * 32), 0);

			//DrawFontText(to_string(y).c_str(), 220.0f, 56 + (y * 32), 0.0f, FALSE, 1);
		}

	glEnd();

		// re-enable textures
	glEnable(GL_TEXTURE_2D);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glPopMatrix();
}

void gfx_DrawGoalLine()
{
	glPushMatrix();

	glDisable(GL_TEXTURE_2D);
	glColor4f(1.0f, 0.3f, 0.3f, 0.75f);

	glLineStipple(1, 0x00FF); 
	glEnable(GL_LINE_STIPPLE);
	glLineWidth(5.0f);

	glBegin(GL_LINES);
		glVertex3f(240, 440 - (GoalHeight * 32), 0);
		glVertex3f(560, 440 - (GoalHeight * 32), 0);
	glEnd();

	glDisable(GL_LINE_STIPPLE);
	
	glLineWidth(1.0f);
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();
}

void gfx_DrawLabels()
{
	static char LabelString[512];

	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, labels->textureID);

	glTranslatef(600.0f, 10.0f, 0.0f);

	// Begin rendering quads
	glBegin(GL_QUADS);
	
		glTexCoord2f(0.0	, 0.0);
        glVertex3f	(0		, 0		, 0);
        glTexCoord2f(0.0	, 1);
        glVertex3f	(0		, 128	, 0);
		glTexCoord2f(1		, 1);
        glVertex3f	(128	, 128	, 0);
        glTexCoord2f(1		, 0.0);
        glVertex3f	(128	, 0		, 0);

	glEnd();

	glPopMatrix();

	sprintf_s(LabelString, 512, "%d of %d", LinesCleared, LinesRequired);

	DrawFontText(LabelString, 645.0f, 42.0f, 0.0f, FALSE, 1);

	sprintf_s(LabelString, 512, "level %d - %d", CurrentLevel, CurrentScore);

	DrawFontText(LabelString, 645.0f, 106.0f, 0.0f, FALSE, 1);

}

void gfx_DrawExclaim()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, symbols->textureID);

	glTranslatef(guy->locX + 8, guy->locY - 32, 0.0f);

	// Begin rendering quads
	glBegin(GL_QUADS);
	
		glTexCoord2f(0.25	, 0.0);
        glVertex3f	(0		, 0		, 0);
        glTexCoord2f(0.25	, 1);
        glVertex3f	(0		, 32	, 0);
		glTexCoord2f(0.50	, 1);
        glVertex3f	(32		, 32	, 0);
        glTexCoord2f(0.50	, 0.0);
        glVertex3f	(32		, 0		, 0);

	glEnd();

	glPopMatrix();
}

void gfx_DrawLineClear()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, symbols->textureID);

	glTranslatef(400, (int)(56 + (lineClearYpos * 32) - (50 - lineClearCounter)), 0.0f);

	// Begin rendering quads
	glBegin(GL_QUADS);
	
		glTexCoord2f(0.50	, 0.0);
        glVertex3f	(0		, 0		, 0);
        glTexCoord2f(0.50	, 1);
        glVertex3f	(0		, 32	, 0);
		glTexCoord2f(1.0	, 1);
        glVertex3f	(64		, 32	, 0);
        glTexCoord2f(1.0	, 0.0);
        glVertex3f	(64		, 0		, 0);

	glEnd();

	glPopMatrix();

	return;
}

void gfx_DrawIndicator()
{
	static float WaveShift = 0;
	static float sineOsc = 0;

	sineOsc += 0.2;

	if (sineOsc > 2 * M_PI)
		sineOsc = 0;

	WaveShift = sin(sineOsc) * 8;

	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, symbols->textureID);

	glTranslatef(200 + WaveShift, 424 - (GoalHeight * 32), 0.0f);

	// Begin rendering quads
	glBegin(GL_QUADS);
	
		glTexCoord2f(0		, 0.0);
        glVertex3f	(0		, 0		, 0);
        glTexCoord2f(0		, 1);
        glVertex3f	(0		, 32	, 0);
		glTexCoord2f(0.25	, 1);
        glVertex3f	(32		, 32	, 0);
        glTexCoord2f(0.25	, 0.0);
        glVertex3f	(32		, 0		, 0);

	glEnd();

	glPopMatrix();

	return;
}

void gfx_DrawWater()
{
	static float WaveShift = 0;
	static float sineOsc = 0;

	sineOsc += 0.05;

	if (sineOsc > 2 * M_PI)
		sineOsc = 0;

	WaveShift = sin(sineOsc) * 16;

	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, water->textureID);

	glTranslatef(WaveShift, 0.0f, 0.0f);

	//Begin rendering quads
	glBegin(GL_QUADS);
	
		glTexCoord2f(0, 0.0);
        glVertex3f(-16, 480 - WaterLevel, 0);
        glTexCoord2f(0, 0.25);
        glVertex3f(-16, 480 - WaterLevel + 8, 0);
		glTexCoord2f(26, 0.25);
        glVertex3f(816, 480 - WaterLevel + 8, 0);
        glTexCoord2f(26, 0.0);
        glVertex3f(816, 480 - WaterLevel, 0);

	glEnd();

	glBegin(GL_QUADS);

		glTexCoord2f(0, 0.25);
		glVertex3f(-16, 480 - WaterLevel + 8, 0);
		glTexCoord2f(0, 1.0f);
		glVertex3f(-16, 480, 0);
		glTexCoord2f(0, 1.0f);
		glVertex3f(816, 480, 0);
		glTexCoord2f(0, 0.25);
		glVertex3f(816, 480 - WaterLevel + 8, 0);

	glEnd();
	
	glPopMatrix();
}

void gfx_DrawTalkyBubble(char * talkies, float x, float y)
{
	static int textlength;

	x += 8;

	textlength = GetFontTextLength(talkies);

	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, talkybubble->textureID);

	glTranslatef((int)x, (int)y, 0.0f);

	// Draw 3 quads

	glBegin(GL_QUADS);
	
		glTexCoord2i(0.0, 0.0);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0.0, 1.0);
        glVertex3i(0, 32, 00);
		glTexCoord2f(0.5, 1.0);
        glVertex3i(16, 32, 0);
        glTexCoord2f(0.5, 0.0);
        glVertex3i(16, 0, 0);

	glEnd();

	// middle piece

	glBegin(GL_QUADS);
	
		glTexCoord2f(0.50, 0.0);
        glVertex3i(16, 0, 0);
        glTexCoord2f(0.50, 1.0);
        glVertex3i(16, 32, 00);
		glTexCoord2f(0.75, 1.0);
        glVertex3i(textlength + 8, 32, 0);
        glTexCoord2f(0.75, 0.0);
        glVertex3i(textlength + 8, 0, 0);

	glEnd();
	
	glBegin(GL_QUADS);
	
		glTexCoord2f(0.75, 0.0);
        glVertex3i(textlength + 8, 0, 0);
        glTexCoord2f(0.75, 1.0);
        glVertex3i(textlength + 8, 32, 0);
		glTexCoord2f(1.0, 1.0);
        glVertex3i(textlength + 16, 32, 0);
        glTexCoord2f(1.0, 0.0);
        glVertex3i(textlength + 16, 0, 0);

	glEnd();
	
	DrawFontText(talkies, 8.0f, 9.0f, 0.0f, FALSE, 1);

	glPopMatrix();

	return;
}

void gfx_DrawBlocks()
{
	static int x,y;
	static cBlock temp;
	// First draw the static blocks
	
	for (x = 0; x < 10; x++)
	{
		for (y = 0; y < 12; y++)
		{
			temp = field[x][y];

			if (temp.Active == 1)
			{
				glPushMatrix();
				
				glBindTexture(GL_TEXTURE_2D, block->textureID);
				
				glTranslatef(240 + (x * 32), 56 + (y * 32), 0);

				glBegin(GL_QUADS);
	
					glTexCoord2f((0.25 * temp.Type), 0);
					glVertex3i(0, 0, 0);
					glTexCoord2f((0.25 * temp.Type), 1);
					glVertex3i(0, 32, 00);
					glTexCoord2f(0.25 + (0.25 * temp.Type), 1);
					glVertex3i(32, 32, 0);
					glTexCoord2f(0.25 + (0.25 * temp.Type), 0);
					glVertex3i(32, 0, 0);

				glEnd();

				glPopMatrix();
			}
		}
	}
}

void gfx_DrawPiece()
{
	int i, j;

	for (i = 0; i < 5; i++)
	{
		for (j = 0; j < 5; j++)
		{
			if (pieces.GetBlockType (PieceType, PieceRotation, j, i) != 0)
			{
				glPushMatrix();
				
				glBindTexture(GL_TEXTURE_2D, block->textureID);
				
				glTranslatef(240 + (currPosX * 32), 56 + (currPosY * 32), 0);

				glBegin(GL_QUADS);
	
					glTexCoord2f(0.75, 0);
					glVertex3i((i * 32), (j * 32), 0);
					glTexCoord2f(0.75, 1);
					glVertex3i((i * 32), (j * 32) + 32, 00);
					glTexCoord2f(1, 1);
					glVertex3i((i * 32) + 32, (j * 32) + 32, 0);
					glTexCoord2f(1, 0);
					glVertex3i((i * 32) + 32, (j * 32), 0);

				glEnd();

				glPopMatrix();				

			}
		}
	}

	for (i = 0; i < 5; i++)
	{
		for (j = 0; j < 5; j++)
		{
			if (pieces.GetBlockType (NextPieceType, NextPieceRotation, j, i) != 0)
			{
				glPushMatrix();
				
				glBindTexture(GL_TEXTURE_2D, block->textureID);
				
				glTranslatef(625, 120, 0);

				glBegin(GL_QUADS);
	
					glTexCoord2f(0.75, 0);
					glVertex3i((i * 16), (j * 16), 0);
					glTexCoord2f(0.75, 1);
					glVertex3i((i * 16), (j * 16) + 16, 0);
					glTexCoord2f(1, 1);
					glVertex3i((i * 16) + 16, (j * 16) + 16, 0);
					glTexCoord2f(1, 0);
					glVertex3i((i * 16) + 16, (j * 16), 0);

				glEnd();

				glPopMatrix();				

			}
		}
	}

	return;
}

/*
void gfx_DrawImageRot(GLuint srcimg, int spritew, int spriteh, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh, float rotation, int flipped)
{
	// Create some texturing coordinates
	// Because they need to be between 0 and 1
	float topx = ((float)sx / (float)spritew);
	float topy = ((float)sy / (float)spriteh);
	float lowx = topx + ((float)sw / (float)spritew); // this is a little tricky, meow.
	float lowy = topy + ((float)sh / (float)spriteh);

	glLoadIdentity(); // Reset the matrix! "Woah... no way."

	// Enable texturing
	glEnable(GL_TEXTURE_2D);
 
	// Load the texture
	glBindTexture(GL_TEXTURE_2D, srcimg);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Blending, alpha-style.

	// move to where we are going to draw ..
	glTranslatef((float)dx, (float)dy, 0.0f);

	// move it back (un-doing the center of rotation move below)
	glTranslatef((float)(dw / 2), (float)(dh / 2), 0.0f);

	// rotation!
	if (rotation != 0)
		glRotatef(rotation,0.0f,0.0f,1.0f);	

	if (flipped == 1)
		glRotatef(180, 0.0f, 1.0f, 0.0f);

	// move to the center for rotation (remember that rotates and translates are teh backwards executing!)
	glTranslatef(-(float)(dw / 2), -(float)(dh / 2), 0.0f);


	// color the quad white (no strange tinting)
	glColor4ub(255, 255, 255, 255);
	
	//some debugging junk
	//printf("\nsx\tsy\tsw\tsh\n%i\t%i\t%i\t%i",sx,sy,sw,sh);
	//printf("\ntopx\ttopy\tlowx\tlowy\n%f\t%f\t%f\t%f\n",topx,topy,lowx,lowy);

	glBegin(GL_QUADS);
		//Top-left vertex (corner)
		glTexCoord2f(topx, topy);
		glVertex3f(0, 0, 0.0f); //need to convert to floats, because they are OpenGL's favorite.
	
		//Bottom-left vertex (corner)
		glTexCoord2f(lowx, topy);
		glVertex3f((float)(dw), 0, 0.0f);
	
		//Bottom-right vertex (corner)
		glTexCoord2f(lowx, lowy);
		glVertex3f((float)(dw), (float)(dh), 0.0f);
	
		//Top-right vertex (corner)
		glTexCoord2f(topx, lowy);
		glVertex3f(0, (float)(dh), 0.0f);
	glEnd();
 
	//Disable texturing
	glDisable(GL_TEXTURE_2D);
}
*/
void gfx_DrawWinScreen()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, winlose->textureID);
	
	glTranslatef(144.0f, 112.0f - (100 - lineClearCounter), 0.0f);

	//Begin rendering quads
    glBegin(GL_QUADS);
	
		glTexCoord2i(0, 0);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0, 0.5);
        glVertex3i(0, 256, 00);
		glTexCoord2f(1, 0.5);
        glVertex3i(512, 256, 0);
        glTexCoord2f(1, 0);
        glVertex3i(512, 0, 0);

	glEnd();
	
	glPopMatrix();

	return;
}

void gfx_DrawLoseScreen()
{
	glPushMatrix();

	glBindTexture(GL_TEXTURE_2D, winlose->textureID);
	
	glTranslatef(144.0f, 112.0f - (100 - lineClearCounter), 0.0f);

	//Begin rendering quads
    glBegin(GL_QUADS);
	
		glTexCoord2f(0, 0.5);
        glVertex3i(0, 0, 0);
        glTexCoord2f(0, 1);
        glVertex3i(0, 256, 00);
		glTexCoord2f(1, 1);
        glVertex3i(512, 256, 0);
        glTexCoord2f(1, 0.5);
        glVertex3i(512, 0, 0);

	glEnd();
	
	glPopMatrix();

	return;
}

void gfx_DrawDarkOverlay()
{
	glPushMatrix();

	glDisable(GL_TEXTURE_2D);

	glColor4f(0.0f, 0.0f, 0.0f, 0.4f);

	glBegin(GL_QUADS);
	
		//glTexCoord2f(0.75, 0);
		glVertex3i(0, 0, 0);
		//glTexCoord2f(0.75, 1);
		glVertex3i(0, 480, 0);
		//glTexCoord2f(1, 1);
		glVertex3i(800, 480, 0);
		//glTexCoord2f(1, 0);
		glVertex3i(800, 0, 0);

	glEnd();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glEnable(GL_TEXTURE_2D);

	glPopMatrix();
}

void gfx_DrawScene(float interpolation)
{
	//static char MouseString[512];

	if (TexturesLoaded == 0)
		gfx_LoadTextures();

	// interpolation is a value from 0 to 1

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	// Draw Level
	gfx_DrawBackground();

	if (InputState.MenuMode == FALSE)
	{
		gfx_DrawLines();

	
		gfx_DrawBlocks();
		gfx_DrawPiece();

		gfx_DrawGoalLine();

		if (LinesCleared >= LinesRequired)
			gfx_DrawIndicator();

		//if (guy->exclaimCounter > 0)
		//	gfx_DrawExclaim();

		guy->Draw();

		gfx_DrawWater();
		//gfx_DebugCrap();

		gfx_DrawLabels();

		if (ShowWinScreen == TRUE || ShowLoseScreen == TRUE)
		{
			gfx_DrawDarkOverlay();

			if (ShowWinScreen == TRUE)
			{
				gfx_DrawWinScreen();
			}
			else if (ShowLoseScreen == TRUE)
			{
				gfx_DrawLoseScreen();
			}
		}
		else
		{
			if (lineClearCounter > 0)
				gfx_DrawLineClear();
		}	

	}
	else // draw menu stuff
	{
		gfx_DrawMenuTitle();
		gfx_DrawWater();
	}

	SDL_GL_SwapBuffers();

	return;
}

#endif