#ifndef _INIT_C_
#define _INIT_C_

// System Includes
#include <windows.h>
#include "SDL.h"
#include "SDL_opengl.h"
#include "SDL_mixer.h"
#include "physfs/physfs.h"

// Local Includes
#include "common.h"
#include "input.h"
#include "extensions.h"

// Some defaults for screen dimensions and other things
#define DEFAULT_SCREEN_WIDTH    800
#define DEFAULT_SCREEN_HEIGHT   480
#define DEFAULT_SCREEN_BPP      32
#define DEFAULT_FULLSCREEN		0
#define DEFAULT_MAX_FPS			75

int InitInput()
{
	// Default values for input struct
	InputState.NeedToShutDown = FALSE;
	InputState.ESCPressed = FALSE;
	InputState.MoveDown = FALSE;
	InputState.MoveLeft = FALSE;
	InputState.MoveRight = FALSE;
	InputState.TryRotate = FALSE;
	InputState.MenuMode = TRUE;

	//InputState.MousePosX = 0;
	//InputState.MousePosY = 0;

	return TRUE;
}

int Configuration(int argc, char **argv)
{
	// Variables
	int i; // for counting args
	int counter;

	// Apply defaults
	ScreenWidth = DEFAULT_SCREEN_WIDTH;
    ScreenHeight = DEFAULT_SCREEN_HEIGHT;
	GameScreenWidth = DEFAULT_SCREEN_WIDTH;
    GameScreenHeight = DEFAULT_SCREEN_HEIGHT;
	ScreenBPP = DEFAULT_SCREEN_BPP;
	Fullscreen = DEFAULT_FULLSCREEN;
	maxFPS = DEFAULT_MAX_FPS;
	
	#ifdef WIN32
        // This is scary win32api stuff that grabs the current display resolution and bit depth
        // Creating a fullscreen window using this method makes for a very short delay when setting the videomade.
		// Short delays = win
        if (Fullscreen)
        {
			// Win32api junk. Hold on tight!
			ScreenWidth = GetSystemMetrics(SM_CXSCREEN);
            ScreenHeight = GetSystemMetrics(SM_CYSCREEN);
			// Phew, it's over.

			ScaleFactor = 1;
			
			if ((ScreenWidth / ScreenHeight) < 1.5)
			{
				counter = DEFAULT_SCREEN_WIDTH;

				while (GameScreenWidth + counter <= ScreenWidth)
				{
					counter += DEFAULT_SCREEN_WIDTH;
					ScaleFactor += 1;
				}
			}
			else
			{
				counter = DEFAULT_SCREEN_HEIGHT;

				while (GameScreenHeight + counter <= ScreenHeight)
				{
					counter += DEFAULT_SCREEN_HEIGHT;
					ScaleFactor += 1;
				}
			}
        }
    #else
        // Is there a linux way to get the screen res? meow.
        if (fullscreen)
        {
            //ScreenWidth = SCREEN_WIDTH;
            //ScreenHeight = SCREEN_HEIGHT;
        }
    #endif
	
	// TODO: Does config file exist?

	// TODO: Load config file over defaults

	// Apply commandline settings over config/defaults
	
	for (i = 1; i < argc; i++)
	{
		if ((strcmp(argv[i], "--fullscreen") == 0) || (strcmp(argv[i], "-f") == 0))
		{
			Log("Fullscreen enabled via commandline!");
			Fullscreen = TRUE;
		}
		else if ((strcmp(argv[i], "--help") == 0) || (strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "/?") == 0))
		{
			Log("Some commandline arguments:\n\t-f --fullscreen\t\tEnable fullscreen mode\n\t-n --nosound\t\tDisable sound\n\t-j --joystick\t\tEnable joystick\n");
			return FALSE;
		}
		else if ((strcmp(argv[i], "--version") == 0) || (strcmp(argv[i], "-v") == 0))
		{
			printf("This is fydo's LD17 entry, version %s!", systemVersion);
			Log("This is fydo's LD17 entry, version %s!", systemVersion);
			return FALSE;
		}
		else if ((strcmp(argv[i], "--nosound") == 0) || (strcmp(argv[i], "-n") == 0))
		{
			Log("The nosound option hasn't been implemented yet! Sorry!");
			// no sound!
			//return FALSE;
		} 
		else if ((strcmp(argv[i], "--joystick") == 0) || (strcmp(argv[i], "-j") == 0))
		{
			Log("The joystick stuff hasn't been implemented yet! Sorry!");
			// joystick!
			//return FALSE;
		}
		else
		{
			// bad option
			Log("Commandline argument unknown: (%s) ... Ignoring.", argv[i]);
		}
	}

	// Compute config stuff
	skipMSPerFrame = (1000.0f / (maxFPS));

	return TRUE;
}

void SetCaption()
{
	char caption[120];

	// Set up a caption for the window
	strcpy_s (caption, 120, "fydo's LD17 entry v");
	strcat_s (caption, 120, systemVersion);
	strcat_s (caption, 120, " - http://fydo.net");
	
    SDL_WM_SetCaption(caption, NULL);
	
	return;
}

int InitSDLThings()
{
	// Variables
	SDL_Surface *surface;
	int audio_rate = 22050;
	Uint16 audio_format = MIX_DEFAULT_FORMAT;
	int audio_channels = 2;
	int audio_buffers = 1024;
	int videoFlags;
	const SDL_VideoInfo *videoInfo;
	int ScreenBPP;

	Log("Initializing SDL (video + sound) ... ");
	
	// Lets get SDL going, shall we?
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0 ) 
    {
        Log("!!! Unable to init SDL: %s", SDL_GetError());
        return FALSE;
    }

	if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) != 0) {
		Log("!!! Unable to initialize audio: %s", Mix_GetError());
		return FALSE;
	}

	Log("Done!");
    
	SetCaption();

	// Hide the cursor while it is over the app
	//SDL_ShowCursor(SDL_DISABLE);

    videoInfo = SDL_GetVideoInfo();

    if (!videoInfo)
    {
    	Log("!!! Video query failed: %s\n", SDL_GetError());
        return FALSE;
    }

    videoFlags  = SDL_OPENGL;			// Enable OpenGL in SDL
    videoFlags |= SDL_GL_DOUBLEBUFFER;	// Enable double buffering
	videoFlags |= SDL_GL_SWAP_CONTROL;	// Enable Vsync
    videoFlags |= SDL_HWPALETTE;		// Store the palette in hardware
    //videoFlags |= SDL_RESIZABLE;		// Enable window resizing 

    if ( videoInfo->hw_available )
        videoFlags |= SDL_HWSURFACE;
    else
        videoFlags |= SDL_SWSURFACE;

    // This checks if hardware blits can be done
    if ( videoInfo->blit_hw )
        videoFlags |= SDL_HWACCEL;

    if (Fullscreen) 
		videoFlags |= SDL_FULLSCREEN;

    ScreenBPP = videoInfo->vfmt->BitsPerPixel;

    Log("Setting video mode: %dx%d %dbpp ... ", ScreenWidth, ScreenHeight, ScreenBPP);
	
    surface = SDL_SetVideoMode(ScreenWidth, ScreenHeight, ScreenBPP, videoFlags);

    // Verify that it worked
    if ( !surface )
    {
    	Log("!!! Video mode set failed: %s\n", SDL_GetError( ) );
        return FALSE;
    }

	Log("Done!");

	// Let's render a black screen so the display will come up faster
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	SDL_GL_SwapBuffers();

	return TRUE;
}

int SetupOpenGL()
{
	// viewport / ortho stuff
	glViewport(0, 0, GameScreenWidth, GameScreenHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	//SetPerspective(POV, (GLfloat)ScreenWidth/(GLfloat)ScreenHeight, 0.1f, 150.0f);
	//SetPerspective(45.0f, (GLfloat)ScreenWidth/(GLfloat)ScreenHeight, 0.1f, 100.0f);
	glOrtho(0, GameScreenWidth, GameScreenHeight, 0, 0, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// texturing
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	// lighting
	//glEnable(GL_LIGHTING);
	glDisable(GL_LIGHTING);
    
	// smooth shading
	glShadeModel( GL_SMOOTH );

	// 4-byte pixel alignment
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

	// background color
	glClearColor( 0.9f, 0.9f, 0.9f, 1.0f );
	glClearDepth(1.0f);

	// depth testing
	glDisable( GL_DEPTH_TEST );
	//glEnable( GL_DEPTH_TEST );
	//glDepthFunc(GL_LEQUAL);
 
	// yay alphablends
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	// don't draw faces we can't see :)
	glEnable(GL_CULL_FACE);

	glPointSize(1.0f);
	glLineWidth(1.0f);

	// track material ambient and diffuse from surface color, call it before glEnable(GL_COLOR_MATERIAL)
	//glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	//glEnable(GL_COLOR_MATERIAL);
    
	// sure, why not?
	glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
	
	// FSAA
	//SDL_GL_SetAttribute( SDL_GL_MULTISAMPLEBUFFERS, 1 ) ;
	//SDL_GL_SetAttribute( SDL_GL_MULTISAMPLESAMPLES, 8 ) ;

	return ExtensionCheck();

	//gfx_LoadTextures(); // init textures

	return TRUE;
}

int InitPhysFS(char *arg)
{
	// Get PhysFS stuff going
	if (!PHYSFS_init(arg))
	{
		Log("!!! PhysicsFS Error: %s", PHYSFS_getLastError());
		return FALSE;
	}
	if (PHYSFS_setSaneConfig("fydo", "LD17", "ZIP", 0, 0) == 0)
	{
		Log("!!! PhysicsFS Error: %s", PHYSFS_getLastError());
		return FALSE;
	}
	
	//if (!PHYSFS_mount("./data.7z", NULL, 1))
	//if (!PHYSFS_mount("./data.zip", NULL, 1))
	//if (!PHYSFS_addToSearchPath("./data.zip", 1))
	//    fprintf(stderr, PHYSFS_getLastError());
	
	return TRUE;
}

#endif