#ifndef _TEXTURE_C_
#define _TEXTURE_C_

//#include <stdio.h>
//#include <string>
#include <string.h>
#include <stdlib.h>
//#include "SDL.h"
#include "SDL_opengl.h"
//#include "GL/glext.h"
#include "physfs/physfs.h"

#include "common.h"
#include "texture.h"
#include "extensions.h"

// Borrowed the following structs from the WINE project (thanks a bunch!)
// --- Start cool WINE structs

typedef unsigned long   DWORD;
typedef unsigned short  WORD;
typedef long            LONG;
//typedef void            VOID;
typedef VOID           *LPVOID;

typedef struct
{
	DWORD       dwColorSpaceLowValue;   // low boundary of color space that is to
										// be treated as Color Key, inclusive
	DWORD       dwColorSpaceHighValue;  // high boundary of color space that is
										// to be treated as Color Key, inclusive
} DDCOLORKEY;
		


//
//DDPIXELFORMAT
//
typedef struct
{
	DWORD       dwSize;                 // size of structure
	DWORD       dwFlags;                // pixel format flags
	DWORD       dwFourCC;               // (FOURCC code)
	union
	{
		DWORD   dwRGBBitCount;          // how many bits per pixel
		DWORD   dwYUVBitCount;          // how many bits per pixel
		DWORD   dwZBufferBitDepth;      // how many total bits/pixel in z buffer (including any stencil bits)
		DWORD   dwAlphaBitDepth;        // how many bits for alpha channels
		DWORD   dwLuminanceBitCount;    // how many bits per pixel
		DWORD   dwBumpBitCount;         // how many bits per "buxel", total
		DWORD   dwPrivateFormatBitCount;// Bits per pixel of private driver formats. Only valid in texture
										// format list and if DDPF_D3DFORMAT is set
	} x;
	union
	{
		DWORD   dwRBitMask;             // mask for red bit
		DWORD   dwYBitMask;             // mask for Y bits
		DWORD   dwStencilBitDepth;      // how many stencil bits (note: dwZBufferBitDepth-dwStencilBitDepth is total Z-only bits)
		DWORD   dwLuminanceBitMask;     // mask for luminance bits
		DWORD   dwBumpDuBitMask;        // mask for bump map U delta bits
		DWORD   dwOperations;           // DDPF_D3DFORMAT Operations
	} y;
	union
	{
		DWORD   dwGBitMask;             // mask for green bits
		DWORD   dwUBitMask;             // mask for U bits
		DWORD   dwZBitMask;             // mask for Z bits
		DWORD   dwBumpDvBitMask;        // mask for bump map V delta bits
		struct
		{
			WORD    wFlipMSTypes;       // Multisample methods supported via flip for this D3DFORMAT
			WORD    wBltMSTypes;        // Multisample methods supported via blt for this D3DFORMAT
		} MultiSampleCaps;

	} z;
	union
	{
		DWORD   dwBBitMask;             // mask for blue bits
		DWORD   dwVBitMask;             // mask for V bits
		DWORD   dwStencilBitMask;       // mask for stencil bits
		DWORD   dwBumpLuminanceBitMask; // mask for luminance in bump map
	} xx;
	union
	{
		DWORD   dwRGBAlphaBitMask;      // mask for alpha channel
		DWORD   dwYUVAlphaBitMask;      // mask for alpha channel
		DWORD   dwLuminanceAlphaBitMask;// mask for alpha channel
		DWORD   dwRGBZBitMask;          // mask for Z channel
		DWORD   dwYUVZBitMask;          // mask for Z channel
	} xy;
} DDPIXELFORMAT;

//
// DDSCAPS2
// 
typedef struct
{
	DWORD       dwCaps;         // capabilities of surface wanted
	DWORD       dwCaps2;
	DWORD       dwCaps3;
	union
	{
		DWORD       dwCaps4;
		DWORD       dwVolumeDepth;
	} a;
} DDSCAPS2;
	


//
// DDSURFACEDESC2
//
typedef struct _DDSURFACEDESC2
{
		 DWORD   dwSize;         /* 0: size of the DDSURFACEDESC2 structure*/
		 DWORD   dwFlags;        /* 4: determines what fields are valid*/
		 DWORD   dwHeight;       /* 8: height of surface to be created*/
		 DWORD   dwWidth;        /* C: width of input surface*/
		 union {
				 LONG    lPitch;       /*10: distance to start of next line (return value only)*/
				 DWORD   dwLinearSize; /*10: formless late-allocated optimized surface size */
		 } DUMMYUNIONNAME1;
		 DWORD   dwBackBufferCount;/* 14: number of back buffers requested*/
		 union {
				 DWORD   dwMipMapCount;/* 18:number of mip-map levels requested*/
				 DWORD   dwRefreshRate;/* 18:refresh rate (used when display mode is described)*/
				 DWORD   dwSrcVBHandle;/* 18:source used in VB::Optimize */
		 } DUMMYUNIONNAME2;
		 DWORD   dwAlphaBitDepth;/* 1C:depth of alpha buffer requested*/
		 DWORD   dwReserved;     /* 20:reserved*/
		 LPVOID  lpSurface;      /* 24:pointer to the associated surface memory*/
		 union {
				 DDCOLORKEY      ddckCKDestOverlay; /* 28: CK for dest overlay use*/
				 DWORD           dwEmptyFaceColor;  /* 28: color for empty cubemap faces */
		 } DUMMYUNIONNAME3;
		 DDCOLORKEY      ddckCKDestBlt;  /* 30: CK for destination blt use*/
		 DDCOLORKEY      ddckCKSrcOverlay;/* 38: CK for source overlay use*/
		 DDCOLORKEY      ddckCKSrcBlt;   /* 40: CK for source blt use*/
 
		 union {
				 DDPIXELFORMAT   ddpfPixelFormat;/* 48: pixel format description of the surface*/
				 DWORD           dwFVF;  /* 48: vertex format description of vertex buffers */
		 } DUMMYUNIONNAME4;
		 DDSCAPS2        ddsCaps;  /* 68: DDraw surface caps */
		 DWORD           dwTextureStage; /* 78: stage in multitexture cascade */
} DDSURFACEDESC2;
	
#define FOURCC_DXT1	MAKEFOURCC('D', 'X', 'T', '1')
#define FOURCC_DXT3	MAKEFOURCC('D', 'X', 'T', '3')
#define FOURCC_DXT5	MAKEFOURCC('D', 'X', 'T', '5')

static int MAKEFOURCC(int ch0, int ch1, int ch2, int ch3)
{
	return ((int)(DWORD)(ch0)|((int)(DWORD)(ch1) << 8)| ((int)(DWORD)(ch2) << 16) | ((int)(DWORD)(ch3) << 24));
}

// --- End cool WINE structs

typedef struct
{
	GLsizei  width;
	GLsizei  height;
	GLint    components;
	GLenum   format;
	int      numMipMaps;
	GLubyte *pixels;
}  Texture;

//TextureRef_t	 *CurrentRefs; // pointer to an array of TextureRef_ts
TextureRef_t *textures;

int numberOfRefsInArray = 0;
int	numberOfRefsAllocated = 0;

/*
int TextureExistsInArray(const char* filename)
{
	int i = 0;
	
	for (i = 0; i < numberOfRefsInArray; i++)
	{
		if (strcmp(CurrentRefs[i].filename, filename) == 0)
		{
			return i;
		}
	}

	return -1;
}
*/

/*
void AddTextureToArray(TextureRef_t newTexture)
{ 
	void *_tmp;

	if (numberOfRefsInArray == numberOfRefsAllocated) { // need more refs?

		if (numberOfRefsAllocated == 0)
			numberOfRefsAllocated = 4; // start off with 4 refs
		else
			numberOfRefsAllocated *= 2; // double the refs allocated
		
		// Make the reallocation transactional by using a temporary variable first
        _tmp = realloc(CurrentRefs, (numberOfRefsAllocated * sizeof(TextureRef_t)));

		// If the reallocation didn't go so well, inform the user and bail out
        if (!_tmp)
        {
                Log("!!! Couldn't realloc memory!");
                Shutdown(2);
        }

		CurrentRefs = (TextureRef_t*)_tmp;
	}	

	CurrentRefs[numberOfRefsInArray] = newTexture; 
	
	numberOfRefsInArray++;

	return;
}
*/

// TODO: Get this to work :D
/*
int RemoveTexture(const char * filename)
{
	// Search through references..
	//int result = -1;
	int i;

	for (i = 0; i < numberOfRefsInArray; i++)
	{
		// If the filename matches remove the reference from the array
		if (filename == CurrentRefs[i].filename)
		{
			// TODO: Remove reference from the array
			return TRUE;
		}
	}
    
	// No filename matches
	return FALSE;
}
*/

Texture* LoadDDSTextureFile(const char * filename)
{
	Texture *pDDSImageData;
	DDSURFACEDESC2 ddsd;
	char filecode[4];
	int factor;
	int bufferSize;

	const int dxt1 = FOURCC_DXT1;
	const int dxt3 = FOURCC_DXT3;
	const int dxt5 = FOURCC_DXT5;

	PHYSFS_file* pFile;

	if (PHYSFS_exists(filename) == 0)
	{
		Log("!!! Texture doesn't exist: %s", filename);
		Shutdown(2);
	}

	pFile = PHYSFS_openRead(filename);

	//---
    
	// Verify the file is a true .dds file
	PHYSFS_read(pFile, filecode, 1, 4);

	if(strncmp(filecode, "DDS ", 4) != 0)
	{
		Log("!!! The file \"%s\" doesn't appear to be a valid .dds file!", filename);
		PHYSFS_close(pFile);
		Shutdown(2);
	}

	// Get the surface descriptor
	PHYSFS_read( pFile, &ddsd, 1, sizeof(ddsd) );

	pDDSImageData = (Texture*) malloc(sizeof(Texture));

	memset(pDDSImageData, 0, sizeof(Texture));

	//
	// This .dds loader supports the loading of compressed formats DXT1, DXT3 
	// and DXT5.
	//

	factor = 2; // defaults?
    
	if (ddsd.DUMMYUNIONNAME4.ddpfPixelFormat.dwFourCC == dxt1)
	{
		// DXT1's compression ratio is 8:1
		pDDSImageData->format = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
		factor = 2;
            
	} 
	else if (ddsd.DUMMYUNIONNAME4.ddpfPixelFormat.dwFourCC == dxt3)
	{
		// DXT3's compression ratio is 4:1
		pDDSImageData->format = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
		factor = 4;            
	} 
	else if (ddsd.DUMMYUNIONNAME4.ddpfPixelFormat.dwFourCC == dxt5 ) 
	{
		// DXT5's compression ratio is 4:1
		pDDSImageData->format = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
		factor = 4;           
	} 
	else 
	{
		Log("!!! The file \"%s\" doesn't appear to be compressed using DXT1, DXT3, or DXT5!", filename);
		Shutdown(2);
	}

	//
	// How big will the buffer need to be to load all of the pixel data 
	// including mip-maps?
	//

	if (ddsd.DUMMYUNIONNAME1.dwLinearSize == 0)
	{
		Log("!!! Error loading texture \"%s\": dwLinearSize is 0!", filename);
		Shutdown(2);
	}

	if (ddsd.DUMMYUNIONNAME2.dwMipMapCount > 1)
		bufferSize = ddsd.DUMMYUNIONNAME1.dwLinearSize * factor;
	else
		bufferSize = ddsd.DUMMYUNIONNAME1.dwLinearSize;

	pDDSImageData->pixels = (unsigned char*)malloc(bufferSize * sizeof(unsigned char));

	PHYSFS_read(pFile, pDDSImageData->pixels, 1, bufferSize);
    
	// Close the file
	PHYSFS_close(pFile);

	pDDSImageData->width      = ddsd.dwWidth;
	pDDSImageData->height     = ddsd.dwHeight;
	pDDSImageData->numMipMaps = ddsd.DUMMYUNIONNAME2.dwMipMapCount;

	if (ddsd.DUMMYUNIONNAME4.ddpfPixelFormat.dwFourCC == dxt1)
		pDDSImageData->components = 3;
	else
		pDDSImageData->components = 4;

	return pDDSImageData;
}

TextureRef_t * LoadDDSTexture(const char * filename)
{
	Texture *pDDSImageData = LoadDDSTextureFile(filename);

	TextureRef_t * ref;

	int nSize;
	int nOffset = 0;

	int nHeight;//     = pDDSImageData->height;
	int nWidth;//      = pDDSImageData->width;
	int nNumMipMaps;// = pDDSImageData->numMipMaps;
    int nBlockSize;
	GLuint textureID[1];

	int i;

	if( pDDSImageData != NULL )
	{
		nHeight     = pDDSImageData->height;
		nWidth      = pDDSImageData->width;
		nNumMipMaps = pDDSImageData->numMipMaps;

		if( pDDSImageData->format == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT )
			nBlockSize = 8;
		else
			nBlockSize = 16;

		glGenTextures( 1, &textureID[0] );
		glBindTexture( GL_TEXTURE_2D, textureID[0] );

		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

		// Load the mip-map levels
		
		for( i = 0; i < nNumMipMaps; ++i )
		{
			if( nWidth  == 0 ) nWidth  = 1;
			if( nHeight == 0 ) nHeight = 1;

			nSize = ((nWidth+3)/4) * ((nHeight+3)/4) * nBlockSize;
			//fprintf (stdout, "ready to go!\n");
			glCompressedTexImage2DARB( GL_TEXTURE_2D,
										i,
										pDDSImageData->format,
										nWidth,
										nHeight,
										0,
										nSize,
										pDDSImageData->pixels + nOffset );

			nOffset += nSize;

			// Half the image size for the next mip-map level...
			nWidth  = (nWidth  / 2);
			nHeight = (nHeight / 2);
		}

        ref = (TextureRef_t *)malloc(sizeof(TextureRef_t));
		ref->height = nHeight;
		ref->width = nWidth;
		ref->textureID = textureID[0];
		ref->filename = (char*)filename;
		//ref.bpp = pDDSImageData->
        
	}
	else
	{
		Log("!!! Texture could not be loaded: %s\nNow exiting ...\n", filename);
		Shutdown(2);
	}

	if( pDDSImageData != NULL )
	{
		if( pDDSImageData->pixels != NULL )
			free( pDDSImageData->pixels );

		free( pDDSImageData );
	}

	return ref;
}


TextureRef_t * LoadTGATexture(char * filename)
{
	// For flipping texture:
	unsigned char bTemp;
	unsigned char *pLine1, *pLine2;
	int			iLineLen,
				iIndex;

	// Reference that will be returned
	TextureRef_t	*ref;

	// Texture "object"
	Texture		*texture;

	// GL Texture reference
	GLuint		tempref[1];
    
	PHYSFS_file* file;
	
	// Magical Targa junk
	// Targa data type -- uncompressed, RGBA image; element
	// 3 is always 2.
	GLubyte		targaMagic[12] = { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

	GLubyte		fileMagic[12];

	// Header data.
	GLubyte		header[6];
	int			bpp;
	GLuint		bytesPerPixel;

	GLuint		imageSize;
	//GLint      sizeRead;
	GLuint		tmp;
	GLuint		i;

	Log("Loading %s ... ", filename);

	if (PHYSFS_exists(filename) == 0)
	{
		Log("!!! Texture doesn't exist: %s", filename);
		Shutdown(2);
	}

	file = PHYSFS_openRead(filename);

	// Start reading in the header, make sure we
	// have the right Targa data type.  

	if(file == NULL || 
		PHYSFS_read(file, fileMagic, sizeof(fileMagic), 1) 	!= 1 	||
		memcmp(targaMagic, fileMagic, sizeof(targaMagic)) 	!= 0	||
		PHYSFS_read(file, header, sizeof(header), 1)		!= 1 )
	{
		Log("!!! Error loading texture: %s\n", filename);
		PHYSFS_close(file);
		Shutdown(2);
		//return FALSE;
	}
	
	// Determine width and height from header data.

	texture = (Texture*)malloc(sizeof(Texture));

	texture->width  = header[1] * 256 + header[0];
	texture->height = header[3] * 256 + header[2];

	// Determine color bit depth of the image (24 or 32).

	if(texture->width <= 0 || texture->height <= 0 || (header[4] != 24 && header[4] != 32))
	{
		Log("!!! Error loading texture: %s", filename);
		PHYSFS_close(file);
		Shutdown(2);
	}
    
	bpp  = header[4];
	bytesPerPixel = bpp / 8;
	imageSize     = texture->width * texture->height * bytesPerPixel;
	
	//fprintf(stdout, "prepared (%d, %d) - %d... ", texture->width, texture->height, imageSize);
    
	texture->pixels = (GLubyte *)malloc(imageSize);
    	 
	if(texture->pixels == NULL || PHYSFS_read(file, texture->pixels, imageSize, 1) != 1)
	{
		if(texture->pixels != NULL)
			free(texture->pixels);

		Log("!!! Error creating texture: %s", filename);
		PHYSFS_close(file);
		Shutdown(2);		
	} 
	
	// done reading, so close the file

	PHYSFS_close(file);

	for(i = 0; i < imageSize; i += bytesPerPixel)
	{
		tmp = texture->pixels[i];
		texture->pixels[i] = texture->pixels[i + 2];
		texture->pixels[i + 2] = tmp;
	}

	//fprintf(stdout, "referenced ... ");
	// Time to flip the image over!
	// This flipping snippet from http://gpwiki.org/index.php/LoadTGACpp

	iLineLen=texture->width * (bytesPerPixel);
	pLine1=texture->pixels;
	pLine2=&texture->pixels[iLineLen * (texture->height - 1)];

	for( ;pLine1<pLine2;pLine2-=(iLineLen*2))
	{
		for(iIndex=0;iIndex!=iLineLen;pLine1++,pLine2++,iIndex++)
		{
			bTemp=*pLine1;
			*pLine1=*pLine2;
			*pLine2=bTemp;
		}
	}
	//fprintf(stdout, "vflipped ... ");
	// Use OpenGL to load up the texture now

	// Create The Texture
	glGenTextures( 1, &tempref[0] );

	glBindTexture( GL_TEXTURE_2D, tempref[0] );

	// Generate The Texture
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, texture->width, texture->height, 0, GL_RGBA,
		GL_UNSIGNED_BYTE, texture->pixels );

	// Man, I love filtering
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    
	ref = (TextureRef_t *)malloc(sizeof(TextureRef_t));

	ref->textureID = tempref[0];
	//ref->filename = (char*)malloc(strlen(filename) * sizeof(char));
	//ref->filename = (filename);

	ref->height = texture->height;
	ref->width = texture->width;
	//ref.bpp = texture->bpp;

	free(texture->pixels);
	free(texture);

	Log("Done");

	return ref;
}

TextureRef_t * GetTexture(const char * inputfilename, int ForceReload)
{
	//static int result;
	char * newFilename;
	TextureRef_t * temp;

	newFilename = (char *)malloc(sizeof(char) * 1024);

	//newFilename = (char*)malloc(1024 * sizeof(char));

	strcpy_s(newFilename, 1024, "textures/");
	strcat_s(newFilename, 1024, inputfilename);

	// search existing textures first
	temp = textures;
	while (temp)
	{
		if (!strcmp(newFilename, temp->filename))
			return temp;
		temp = temp->next;
	}

	// no texture found, load new one
	if (strstr(inputfilename, ".dds"))
		temp = LoadDDSTexture(&newFilename[0]);
	else if (strstr(inputfilename, ".tga"))
		temp = LoadTGATexture(&newFilename[0]);
	else
		return 0;

	temp->filename = (char *)malloc(sizeof(char) * strlen(newFilename));
	strcpy(temp->filename, newFilename);
	//temp->texid = Tex_LoadTGA(filename);
	temp->next = textures;
	textures = temp;

	free(newFilename);

	return temp;
}

void FreeTextureData()
{
	TextureRef_t *t;
	while (textures)
	{
		t = textures->next;
		free(textures);
		textures = t;
	}

	return;
}

#endif